@@ -9,20 +9,24 @@ found at:
 
 import aiohttp
 import requests
-import json
+
+from auth0.authentication import GetToken
+from auth0.asyncify import asyncify
 from datetime import datetime, timedelta
 from typing import Dict, List, Optional
+from .auth0 import Auth0Client
 from .const import (
     DEVICE_URL,
     LOGIN_URL,
-    BROWSER_USERAGENT,
+    AUTH0_HOST,
     SHARK_APP_ID,
     SHARK_APP_SECRET,
-    SHARK_APP_USERAGENT,
     AUTH0_URL,
     AUTH0_CLIENT_ID,
     AUTH0_SCOPES,
+    BROWSER_USERAGENT,
     EU_DEVICE_URL,
+    EU_AUTH0_HOST,
     EU_LOGIN_URL,
     EU_SHARK_APP_ID,
     EU_SHARK_APP_SECRET,
@@ -31,6 +35,7 @@ from .const import (
 )
 from .exc import SharkIqAuthError, SharkIqAuthExpiringError, SharkIqNotAuthedError
 from .sharkiq import SharkIqVacuum
+from .fallback_auth import FallbackAuth
 
 _session = None
 
@@ -130,6 +135,47 @@ class AylaApi:
             "password": self._password,
             "scope": AUTH0_SCOPES
         }
+    
+    @property
+    def _auth0_login_headers(self) -> Dict[str, Dict]:
+        """
+        Headers for the Auth0 login flow.
+        
+        Returns:
+            A dict containing the headers to send for the Auth0 login flow.
+        """
+        return {
+            "Accept": "application/json, text/plain, */*",
+            "Accept-Encoding": "gzip, deflate, br, zstd",
+            "Accept-Language": "en-US,en;q=0.9",
+            "Content-Type": "application/json",
+            "dnt": "1",
+            "Host": EU_AUTH0_HOST if self.europe else AUTH0_HOST,
+            "Origin": EU_AUTH0_URL if self.europe else AUTH0_URL,
+            "Priority": "u=1, i",
+            "Referrer": EU_AUTH0_URL if self.europe else AUTH0_URL + "/",
+            "Sec-Ch-Ua": '"Chrome";v="137", "Chromium";v="137", "Not A;Brand";v="24"',
+            "Sec-Ch-Ua-Mobile": "?0",
+            "Sec-Ch-Ua-Platform": '"macOS"',
+            "Sec-Fetch-Dest": "empty",
+            "Sec-Fetch-Mode": "cors",
+            "Sec-Fetch-Site": "same-origin",
+            "Sec-Gpc": "1",
+            "User-Agent": BROWSER_USERAGENT
+        }
+    
+    @property
+    def _ayla_login_headers(self) -> Dict[str, Dict]:
+        """
+        Headers for the Ayla login flow.
+        
+        Returns:
+            A dict containing the headers to send for the Ayla login flow.
+        """
+        return {
+            "Content-Type": "application/json",
+            "User-Agent": BROWSER_USERAGENT
+        }
 
     def _set_credentials(self, status_code: int, login_result: Dict):
         """
@@ -158,68 +204,56 @@ class AylaApi:
             login_result: The result of the login response.
         """
         if status_code == 401 and login_result["error"] == "requires_verification":
-            raise SharkIqAuthError(login_result["error_description"] + " (Try logging in with the SharkClean app, then try again)")
+
+            raise SharkIqAuthError(login_result["error_description"] + ". Auth request flagged for verification.")
         elif status_code == 401:
-            raise SharkIqAuthError(login_result["error_description"] + " (Confirm client_id is correct)")
+            raise SharkIqAuthError(login_result["error_description"] + ". Confirm credentials are correct.")
         elif status_code == 400 or status_code == 403:
             raise SharkIqAuthError(login_result["error_description"])
         
         self._auth0_id_token = login_result["id_token"]
 
-    def sign_in(self):
+    async def async_set_cookie(self):
         """
-        Authenticate to Ayla API synchronously.
+        Query Auth0 to set session cookies [required for Auth0 support]
         """
-        auth0_login_data = self._auth0_login_data
-        auth0_headers = {
-            "User-Agent": BROWSER_USERAGENT
-        }
-        api_headers = {
-            "User-Agent": SHARK_APP_USERAGENT
-        }
+        initial_url = self.gen_fallback_url()
+        ayla_client = await self.ensure_session()
 
-        auth0_resp = requests.post(f"{EU_AUTH0_URL if self.europe else AUTH0_URL:s}/oauth/token", json=auth0_login_data, headers=auth0_headers)
-        self._set_id_token(auth0_resp.status_code, auth0_resp.json())
+        async with ayla_client.get(initial_url, allow_redirects=False, headers=self._auth0_login_headers) as auth0_resp:
+            ayla_client.cookie_jar.update_cookies(auth0_resp.cookies)
 
-        login_data = self._login_data
-        resp = requests.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/api/v1/token_sign_in", json=login_data, headers=api_headers)
-        self._set_credentials(resp.status_code, resp.json())
-
-    def refresh_auth(self):
+    async def async_sign_in(self, use_auth0=False):
         """
-        Refresh the authentication synchronously.
+        Authenticate to Ayla API asynchronously via Auth0 [requires cookies]
         """
-        refresh_data = {"user": {"refresh_token": self._refresh_token}}
-        resp = requests.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/users/refresh_token.json", json=refresh_data)
-        self._set_credentials(resp.status_code, resp.json())
+        ayla_client = await self.ensure_session()
 
-    async def async_sign_in(self):
-        """
-        Authenticate to Ayla API asynchronously.
-        """
-        session = await self.ensure_session()
+        if use_auth0 or self.europe:
+            AsyncGetToken = asyncify(GetToken)
+            get_token = AsyncGetToken(EU_AUTH0_HOST if self.europe else AUTH0_HOST, EU_AUTH0_CLIENT_ID if self.europe else AUTH0_CLIENT_ID)
 
-        auth0_login_data = self._auth0_login_data
-        auth0_headers = {
-            "User-Agent": BROWSER_USERAGENT
-        }
-        api_headers = {
-            "User-Agent": SHARK_APP_USERAGENT
-        }
+            auth_result = await get_token.login_async(
+                username=self._email,
+                password=self._password,
+                grant_type='password',
+                scope=AUTH0_SCOPES
+            )
 
-        auth0_url = f"{EU_AUTH0_URL if self.europe else AUTH0_URL}/oauth/token"
-        async with session.post(auth0_url, json=auth0_login_data, headers=auth0_headers) as auth0_resp:
-            auth0_resp_json = await auth0_resp.json()
-            print("Auth status code " + str(auth0_resp.status))
-            print("Auth status code " + json.dumps(auth0_resp_json))
-            self._set_id_token(auth0_resp.status, auth0_resp_json)
+            self._auth0_id_token = auth_result["id_token"]
+        else:
+            auth_result = await Auth0Client.do_auth0_login(
+                ayla_client,
+                self.europe,
+                self._email,
+                self._password
+            )
+            self._auth0_id_token = auth_result["id_token"]
 
         login_data = self._login_data
         login_url = f"{EU_LOGIN_URL if self.europe else LOGIN_URL}/api/v1/token_sign_in"
-        async with session.post(login_url, json=login_data, headers=api_headers) as login_resp:
+        async with ayla_client.post(login_url, json=login_data, headers=self._ayla_login_headers) as login_resp:
             login_resp_json = await login_resp.json()
-            print("Login status code " + str(login_resp.status))
-            print("Login status code " + json.dumps(login_resp_json))
             self._set_credentials(login_resp.status, login_resp_json)
 
 
@@ -227,9 +261,9 @@ class AylaApi:
         """
         Refresh the authentication synchronously.
         """
-        session = await self.ensure_session()
         refresh_data = {"user": {"refresh_token": self._refresh_token}}
-        async with session.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/users/refresh_token.json", json=refresh_data) as resp:
+        ayla_client = await self.ensure_session()
+        async with ayla_client.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/users/refresh_token.json", json=refresh_data, headers=self._ayla_login_headers) as resp:
             self._set_credentials(resp.status, await resp.json())
 
     @property
@@ -251,22 +285,24 @@ class AylaApi:
         self._refresh_token = None
         self._auth_expiration = None
 
-    def sign_out(self):
-        """
-        Sign out and invalidate the access token.
-        """
-        requests.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/users/sign_out.json", json=self.sign_out_data)
-        self._clear_auth()
-
     async def async_sign_out(self):
         """
         Sign out and invalidate the access token.
         """
-        session = await self.ensure_session()
-        async with session.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/users/sign_out.json", json=self.sign_out_data) as _:
+        ayla_client = await self.ensure_session()
+        async with ayla_client.post(f"{EU_LOGIN_URL if self.europe else LOGIN_URL:s}/users/sign_out.json", json=self.sign_out_data) as _:
             pass
         self._clear_auth()
 
+    def gen_fallback_url(self):
+        """
+        Generate a URL for the fallback authentication flow.
+        
+        Returns:
+            The URL for the fallback authentication flow.
+        """
+        return FallbackAuth.GenerateFallbackAuthURL(self.europe)
+
     @property
     def auth_expiration(self) -> Optional[datetime]:
         """
@@ -381,9 +417,11 @@ class AylaApi:
         Returns:
             The response from the request.
         """
-        session = await self.ensure_session()
+        ayla_client = await self.ensure_session()
         headers = self._get_headers(kwargs)
-        return session.request(http_method, url, headers=headers, **kwargs)
+        result = ayla_client.request(http_method, url, headers=headers, **kwargs)
+
+        return result
 
     def list_devices(self) -> List[Dict]:
         """
@@ -444,3 +482,14 @@ class AylaApi:
                 await device.async_get_metadata()
                 await device.async_update()
         return devices
+    
+    async def async_close_session(self):
+        """
+        Close the shared aiohttp ClientSession.
+
+        This should be called when you are finished with the AylaApi object.
+        """
+        shared_session = self.ensure_session()
+        if shared_session is not None:
+            shared_session.close()
+
